/* define the hash table functions */

#include <stdlib.h>
#include <limits.h>

#include "reporter.h"
#include "hash.h"

struct HashTbl {
  int TableSize;		/* actual size of the hash table */
  List *TheLists;		/* array of list header pointers */
  Comparator *eCompare;		/* determine if two elements are equal */
  Hasher *eHash;		/* calculate the hash index for an element */
};

/* replace "Anonymous Author" with "your name" */

char *getHashAuthor(void) {
  return "Anonymous Author";
}

/*
Print some useful information about the current state of the hash table.
This function assumes that there are members of the struct HashTbl named
  TableSize - the number of slots in the Hash table
  TheLists - a pointer to the array of pointers to List headers
If you use different names in your definition of struct HashTbl, you 
will need to update this function.
*/
void PrintHashTableStatistics(HashTable H, ElementPrintLabel P){
  int i;
  int elementCount;		/* number of elements in one list */
  int slotCount;		/* number of slots actually used */
  int totalElementCount;	/* total number of elements in all lists */
  int minLength;		/* list lengths */
  int maxLength;
  int minLocation;		/* index of shortest list */
  int maxLocation;		/* index of longest list */
  int minOccurrence;
  int maxOccurrence;
  Position pos;			/* a single list entry */
  ElementType e;		/* an Element from a list entry */

  totalElementCount = 0;
  slotCount = 0;
  minLength = INT_MAX;
  minLocation = 0;
  minOccurrence = 0;
  maxLength = INT_MIN;
  maxLocation = 0;
  maxOccurrence = 0;

  for (i=0; i<H->TableSize; i++) {
    elementCount = CountListEntries((H->TheLists)[i]);

    if (elementCount < minLength) {
      minLength = elementCount;
      minLocation = i;
      minOccurrence = 1;
    } else if (elementCount == minLength) {
      minOccurrence++;
    }

    if (elementCount > maxLength) {
      maxLength = elementCount;
      maxLocation = i;
      maxOccurrence = 1;
    } else if(elementCount == maxLength) {
      maxOccurrence++;
    }

    if (elementCount > 0) {
      /*  printf(" %4i entries at hash index %6i.\n",elementCount,i); */
      slotCount++;
      totalElementCount += elementCount;
    }
  }
  printf("%5i Number of elements in hash table\n",totalElementCount);
  printf("%5i Number of slots in hash table\n",H->TableSize);
  if (totalElementCount != 0) {
    printf("%5.2f Expected average list length\n",(totalElementCount < H->TableSize) ? 1 : (double)totalElementCount / H->TableSize);
    printf("%5i Number of lists with entries\n",slotCount);
    printf("%5i Minimum list length (%i occurrences)\n",minLength,minOccurrence);
    printf("%5i Maximum list length (%i occurrences)\n",maxLength,maxOccurrence);
    printf("%5.2f Actual average list length\n",(double)totalElementCount/slotCount);
    printf("      Max length list: ");
    pos = GetFirstListEntry(H->TheLists[maxLocation]);
    elementCount = 0;
    while (pos != NULL) {
      if (elementCount < 10) {
	e = GetListEntryElement(pos);
	(*P)(e);
	printf(" ");
      } else {
	printf("...");
	break;
      }
      elementCount++;
      pos = AdvanceListEntry(pos);
    }
    printf("\n");
  }
  printf("\n");

}
