/*

Read a symbol table file and create Hash tables from it.
Print information about the Hash tables.
Input from stdin or named file, output to stdout.

Doug Johnson, djohnson@cs.washington.edu, April 2002 for CSE 373

The format of each row of the input symbol table file is simply

 "<numeric value> <symbol name>"

*/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include "reporter.h"
#include "symbol.h"
#include "hash.h"
#include "symbol_hash.h"

void symbolPrintLabel(void *e);

int main(int argc,char *argv[]) {

  FILE *src;			/* input file */
  long count;			/* number of Symbols defined */
  char *testName;		/* name string for the current test case */
  int testNumber;		/* the number of the current test */
  time_t t;			/* time of day value */

  HashTable H;			/* a HashTable */

  struct Symbol **symbol;	/* array of pointers to Symbol structs */

  /* read from stdin or the named file */

  if (argc == 1) {
    src = stdin;
    printf("Input file name: stdin\n");
  } else if (argc == 2) {
    src = fopen(argv[1],"r");
    if (src == NULL) {
      printf("main: file open error on %s.\n",argv[1]);
      return 1;
    } else {
      printf("Input file name: %s\n",argv[1]);
    }
  } else {  
    printf("usage: %s [<filename>]\n",argv[0]);
    return 1;
  }

  printf("Program author: %s\n",getHashAuthor());
  t = time(NULL);
  printf("Program run: %s\n",ctime(&t));

  /* Read the symbol definitions */

  symbol = readSymbols(src);
  fclose(src);

  /* 
     Build, analyze, and destroy a series of HashTables, as defined by
     the test cases in symbol_hash.c.
 */
  
  testNumber = 0;
  while ((testName=getTestName(testNumber)) != NULL) {
    
    /* Create an empty HashTable */

    H = CreateHashTable(
			getTestTableSize(testNumber),
			getTestHasher(testNumber),
			getTestComparator(testNumber));
    FatalErrorMemory(H);

    /* For each symbol, create an entry in the HashTable */

    count = 0;
    while (symbol[count] != NULL) {
      InsertHashEntry(H,symbol[count]);
      count++;
    }

    /* Tell the user what we saw */

    printf("Test %i: %s\n",testNumber,testName);
    PrintHashTableStatistics(H,symbolPrintLabel);
    
    /* Clean up and do it again */

    DestroyHashTable(H);

    testNumber++;
  }

  /* Release all the memory associated with the Symbols */

  freeSymbols(symbol);

  return 0;
}

/*---------------------------------------------------*/

/* Print a Symbol label. */

void symbolPrintLabel(void *E) {
  if (E==NULL)
    printf("<NULL>");
  else
    printf("%s",((struct Symbol *)E)->name);
}

